/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2007 The LS-Universe Team

  Homepage: http://www.ls-universe.info

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "StdAfx.h"
#include "NetStats.h"
#include <process.h>

#include "winperf.h"
#include "Wininet.h"

extern int iNETUPDATESPEED;		// Value for the statistics timer

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

NetStats::NetStats(int selectedinterface) :
	lastTotalTraffic(0.0),
	lastIncomingTraffic(0.0),
	lastOutGoingTraffic(0.0),

	strAllTotalTraffic("0 KB"),
	strAllIncomingTraffic("0 KB"),
	strAllOutGoingTraffic("0 KB"),

	strCurrentTotalTraffic("0.0 KB/sec"),
	strCurrentIncomingTraffic("0.0 KB/sec"),
	strCurrentOutGoingTraffic("0.0 KB/sec"),

	strMyInterfaceName(""),

	CurrentInterface(-1)
{	
	SelectedInterface = selectedinterface;	// Let us use the first found network connection
	GetInterfaces();

	GetNetworkInterfaceName(&strMyInterfaceName, SelectedInterface);
	
	//Initial Updates 2 times due to inital values
	Update();
	Update();
}

NetStats::~NetStats()
{
	Stop();
}

BOOL NetStats::Start()
{
	DWORD dwThreadId;
	
	hThread = (HANDLE)_beginthreadex(NULL, 0, ThreadProc, this, 0, (UINT*)&dwThreadId);

	return TRUE;
}

void NetStats::Stop()
{
	if(hThread)
	{
		TerminateThread(hThread, 0);
		CloseHandle(hThread);
		hThread = NULL;
	}
}

UINT __stdcall NetStats::ThreadProc(LPVOID pvThis)
{
	NetStats *pThis = (NetStats *) pvThis;

	while(TRUE)
	{
		Sleep(iNETUPDATESPEED);
		pThis->Update();
	}

	return 0;
}


// if you have the Microsoft platform SDK, uncomment the following statement
// and insert Wininet.lib in the linker section of the compiler
#define _I_HAVE_PLATFORM_SDK_INSTALLED_

void NetStats::Update()
{
	const char *units[] = {
		"bytes", "KB", "MB", "GB", "TB", "PB", "EB"
	};
		
	//Initialize Help Variables
	int unit = 0;
	double delta1 = 0;

	double traffic = 0;
	double totaltraffic = 0;

	// Get current Incomingtraffic
	traffic = GetTraffic(SelectedInterface, IncomingTraffic);
	
	delta1 = (double) traffic / 1024.0;
	strCurrentIncomingTraffic.Format("%.1f KB/sec", delta1);

	totaltraffic = GetInterfaceTotalTraffic(SelectedInterface, IncomingTraffic);

	delta1 = (double) totaltraffic;
	unit = 0;
	while ( delta1 > 1024 )
	{
		delta1 = (double) delta1 / 1024.0;
		unit++;
	}
	strAllIncomingTraffic.Format("%.1f %s", delta1, units[unit]);

	// Get current OutGoingtraffic
	traffic = GetTraffic(SelectedInterface, OutGoingTraffic);
		
	delta1 = (double) traffic / 1024.0;
	strCurrentOutGoingTraffic.Format("%.1f KB/sec",delta1);

	totaltraffic = GetInterfaceTotalTraffic(SelectedInterface, OutGoingTraffic);

	delta1 = (double) totaltraffic;
	unit = 0;
	while ( delta1 > 1024 )
	{
		delta1 = (double) delta1 / 1024.0;
		unit++;
	}
	strAllOutGoingTraffic.Format("%.1f %s", delta1, units[unit]);

	// Get current Totaltraffic
	traffic = GetTraffic(SelectedInterface, AllTraffic);
		
	delta1 = (double) traffic / 1024.0;
	strCurrentTotalTraffic.Format("%.1f KB/sec", delta1);

	totaltraffic = GetInterfaceTotalTraffic(SelectedInterface, AllTraffic);

	delta1 = (double) totaltraffic;
	unit = 0;
	while ( delta1 > 1024 )
	{
		delta1 = (double) delta1 / 1024.0;
		unit++;
	}
	strAllTotalTraffic.Format("%.1f %s", delta1, units[unit]);
}

// Little helper functions
// Found them on CodeGuru, but do not know who has written them originally

PERF_OBJECT_TYPE *FirstObject(PERF_DATA_BLOCK *dataBlock)
{
	return (PERF_OBJECT_TYPE *) ((BYTE *)dataBlock + dataBlock->HeaderLength);
}

PERF_OBJECT_TYPE *NextObject(PERF_OBJECT_TYPE *act)
{
	return (PERF_OBJECT_TYPE *) ((BYTE *)act + act->TotalByteLength);
}

PERF_COUNTER_DEFINITION *FirstCounter(PERF_OBJECT_TYPE *perfObject)
{
	return (PERF_COUNTER_DEFINITION *) ((BYTE *) perfObject + perfObject->HeaderLength);
}

PERF_COUNTER_DEFINITION *NextCounter(PERF_COUNTER_DEFINITION *perfCounter)
{
	return (PERF_COUNTER_DEFINITION *) ((BYTE *) perfCounter + perfCounter->ByteLength);
}

PERF_COUNTER_BLOCK *GetCounterBlock(PERF_INSTANCE_DEFINITION *pInstance)
{
	return (PERF_COUNTER_BLOCK *) ((BYTE *)pInstance + pInstance->ByteLength);
}

PERF_INSTANCE_DEFINITION *FirstInstance (PERF_OBJECT_TYPE *pObject)
{
	return (PERF_INSTANCE_DEFINITION *)  ((BYTE *) pObject + pObject->DefinitionLength);
}

PERF_INSTANCE_DEFINITION *NextInstance (PERF_INSTANCE_DEFINITION *pInstance)
{
	// next instance is after
	//    this instance + this instances counter data

	PERF_COUNTER_BLOCK  *pCtrBlk = GetCounterBlock(pInstance);

	return (PERF_INSTANCE_DEFINITION *) ((BYTE *)pInstance + pInstance->ByteLength + pCtrBlk->ByteLength);
}

char *WideToMulti(wchar_t *source, char *dest, int size)
{
	WideCharToMultiByte(CP_ACP, 0, source, -1, dest, size, 0, 0);

	return dest;
}

/*
	returns the traffic of given interface
*/
double NetStats::GetTraffic(int interfaceNumber, int CurrentTrafficType)
{
	try
	{
		#define DEFAULT_BUFFER_SIZE 40960L
			
		CString InterfaceName;
		POSITION pos = Interfaces.FindIndex(interfaceNumber);
		if ( pos == NULL )
			return 0.0;

		InterfaceName = Interfaces.GetAt(pos);
			
		// buffer for performance data
		unsigned char *data = new unsigned char [DEFAULT_BUFFER_SIZE];
		// return value from RegQueryValueEx: ignored for this application
		DWORD type;
		// Buffer size
		DWORD size = DEFAULT_BUFFER_SIZE;
		// return value of RegQueryValueEx
		DWORD ret;
			
		// request performance data from network object (index 510) 
		while ((ret = RegQueryValueExW(HKEY_PERFORMANCE_DATA, L"510", 0, &type, data, &size)) != ERROR_SUCCESS)
		{
			if (ret == ERROR_MORE_DATA) 
			{
				// buffer size was too small, increase allocation size
				size += DEFAULT_BUFFER_SIZE;
					
				delete data;
				data = NULL;
				data = new unsigned char [size];
			} 
			else 
			{
				delete data;
				data = NULL;
				// some unspecified error has occured
				return 0.0;
			}
		}
			
		PERF_DATA_BLOCK *dataBlockPtr = (PERF_DATA_BLOCK *)data;
			
		// enumerate first object of list
		PERF_OBJECT_TYPE *objectPtr = FirstObject(dataBlockPtr);
			
		// trespassing the list 
		for (int a=0 ; a<(int)dataBlockPtr->NumObjectTypes ; a++) 
		{
			char nameBuffer[255] = {0};
				
			// did we receive a network object?
			if (objectPtr->ObjectNameTitleIndex == 510) 
			{
				// Calculate the offset
				DWORD processIdOffset = ULONG_MAX;
				
				// find first counter 
				PERF_COUNTER_DEFINITION *counterPtr = FirstCounter(objectPtr);
				
				// walking trough the list of objects
				for (int b=0 ; b<(int)objectPtr->NumCounters ; b++) 
				{
					// Check if we received datatype wished
					if ((int)counterPtr->CounterNameTitleIndex == CurrentTrafficType)
						processIdOffset = counterPtr->CounterOffset;
						
					// watch next counter
					counterPtr = NextCounter(counterPtr);
				}
					
				if (processIdOffset == ULONG_MAX)
				{
					delete data;
					data = NULL;
					return 0.0;
				}
					
					
				// Find first instance
				PERF_INSTANCE_DEFINITION *instancePtr = FirstInstance(objectPtr);
					
				DWORD fullTraffic = 0.0;
				DWORD traffic = 0.0;
				for (b=0 ; b<objectPtr->NumInstances ; b++) 
				{
					// evaluate pointer to name
					wchar_t *namePtr = (wchar_t *) ((BYTE *)instancePtr + instancePtr->NameOffset);
						
					// get PERF_COUNTER_BLOCK of this instance
					PERF_COUNTER_BLOCK *counterBlockPtr = GetCounterBlock(instancePtr);
						
					// now we have the interface name
						
					char *pName = WideToMulti(namePtr, nameBuffer, sizeof(nameBuffer));
					
					CString iName;
					iName.Format("%s",pName);
					
					if ( CurrentTrafficType == AllTraffic )
					{
						POSITION pos = TotalTraffics.FindIndex(b);
						if (pos != NULL)
						{
							fullTraffic = *((DWORD *) ((BYTE *)counterBlockPtr + processIdOffset));
							TotalTraffics.SetAt(pos,fullTraffic);
						}
					}
					else if ( CurrentTrafficType == IncomingTraffic )
					{
						POSITION pos = TotalIncomingTraffics.FindIndex(b);
						if (pos != NULL)
						{
							fullTraffic = *((DWORD *) ((BYTE *)counterBlockPtr + processIdOffset));
							TotalIncomingTraffics.SetAt(pos,fullTraffic);
						}
					}
					else if ( CurrentTrafficType == OutGoingTraffic )
					{
						POSITION pos = TotalOutGoingTraffics.FindIndex(b);
						if (pos != NULL)
						{
							fullTraffic = *((DWORD *) ((BYTE *)counterBlockPtr + processIdOffset));
							TotalOutGoingTraffics.SetAt(pos,fullTraffic);
						}
					}

					// If the interface the currently selected interface?
					if(InterfaceName == iName)
					{
						traffic = *((DWORD *) ((BYTE *)counterBlockPtr + processIdOffset));
						double acttraffic = (double)traffic;
						double trafficdelta = 0.0;
						// Do we handle a new interface (e.g. due a change of the interface number
						if (CurrentInterface != interfaceNumber)
						{
							if ( CurrentTrafficType == AllTraffic )
							{
								lastTotalTraffic = acttraffic;
							}
							else if ( CurrentTrafficType == IncomingTraffic )
							{
								lastIncomingTraffic = acttraffic;
							}
							else if ( CurrentTrafficType == OutGoingTraffic )
							{
								lastOutGoingTraffic = acttraffic;
							}
							trafficdelta = 0.0;
							CurrentInterface = interfaceNumber;
						}
						else
						{
							if ( CurrentTrafficType == AllTraffic )
							{
								trafficdelta = acttraffic - lastTotalTraffic;
								lastTotalTraffic = acttraffic;
							}
							else if ( CurrentTrafficType == IncomingTraffic )
							{
								trafficdelta = acttraffic - lastIncomingTraffic;
								lastIncomingTraffic = acttraffic;
							}
							else if ( CurrentTrafficType == OutGoingTraffic )
							{
								trafficdelta = acttraffic - lastOutGoingTraffic;
								lastOutGoingTraffic = acttraffic;
							}
						}
						delete data;
						data = NULL;
						return trafficdelta;
					}
					
					// next instance
					instancePtr = NextInstance(instancePtr);
				}
			}
				
			// next object in list
			objectPtr = NextObject(objectPtr);
		}
			
		delete data;
		data = NULL;
		return 0;
	}

	catch(...)
	{
		return 0;
	}
}

/*
	Enumerate installed interfaces. 
	See comments above
*/
BOOL NetStats::GetInterfaces()
{
	try
	{
		#define DEFAULT_BUFFER_SIZE 40960L
		
		Interfaces.RemoveAll();
		unsigned char *data = new unsigned char [DEFAULT_BUFFER_SIZE];
		DWORD type;
		DWORD size = DEFAULT_BUFFER_SIZE;
		DWORD ret;
		
		char s_key[4096] = {0};
		sprintf( s_key , "%d" , 510 );
		
		while ((ret = RegQueryValueExW(HKEY_PERFORMANCE_DATA, L"510", 0, &type, data, &size)) != ERROR_SUCCESS)
		{
			while (ret == ERROR_MORE_DATA) 
			{
				// buffer size was too small, increase allocation size
				size += DEFAULT_BUFFER_SIZE;
					
				delete data;
				data = NULL;
				data = new unsigned char [size];
			} 
			if (ret != ERROR_SUCCESS)
			{
				delete data;
				data = NULL;
				return FALSE;
			}
		}
		
		PERF_DATA_BLOCK	 *dataBlockPtr = (PERF_DATA_BLOCK *)data;
		PERF_OBJECT_TYPE *objectPtr = FirstObject(dataBlockPtr);
		
		for(int a=0 ; a<(int)dataBlockPtr->NumObjectTypes ; a++) 
		{
			char nameBuffer[255] = {0};
			if (objectPtr->ObjectNameTitleIndex == 510) 
			{
				DWORD processIdOffset = ULONG_MAX;
				PERF_COUNTER_DEFINITION *counterPtr = FirstCounter(objectPtr);
				
				for (int b=0 ; b<(int)objectPtr->NumCounters ; b++) 
				{
					if(counterPtr->CounterNameTitleIndex == 520)
						processIdOffset = counterPtr->CounterOffset;
					
					counterPtr = NextCounter(counterPtr);
				}
				
				if (processIdOffset == ULONG_MAX)
				{
					delete data;
					data = NULL;
					return 1;
				}
				
				PERF_INSTANCE_DEFINITION *instancePtr = FirstInstance(objectPtr);
				
				for (b=0 ; b<objectPtr->NumInstances ; b++) 
				{
					wchar_t *namePtr = (wchar_t *) ((BYTE *)instancePtr + instancePtr->NameOffset);
					PERF_COUNTER_BLOCK *counterBlockPtr = GetCounterBlock(instancePtr);
					char *pName = WideToMulti(namePtr, nameBuffer, sizeof(nameBuffer));
					DWORD bandwith = *((DWORD *) ((BYTE *)counterBlockPtr + processIdOffset));				
					DWORD tottraff = 0;

					Interfaces.AddTail(CString(pName));
					Bandwidths.AddTail(bandwith);
					TotalTraffics.AddTail(tottraff);  // initial 0, just for creating the list
					TotalIncomingTraffics.AddTail(tottraff);  // initial 0, just for creating the list
					TotalOutGoingTraffics.AddTail(tottraff);  // initial 0, just for creating the list
					
					instancePtr = NextInstance(instancePtr);
				}
			}
			objectPtr = NextObject(objectPtr);
		}
		delete data;
		data = NULL;
		
		return TRUE;
	}
	catch(...)
	{
		return FALSE;
	}
}

/*
	Returns the count of installed interfaces
*/
int NetStats::GetNetworkInterfacesCount()
{
	return Interfaces.GetCount()-1;
}

/*
	Returns the name of the given interface (-number)
*/
BOOL NetStats::GetNetworkInterfaceName(CString *InterfaceName, int index)
{
	POSITION pos = Interfaces.FindIndex(index);
	if (pos == NULL)
		return FALSE;
	
	InterfaceName->Format("%s",Interfaces.GetAt(pos));

	return TRUE;
}

/*
	Returns bandwith of interface e.g. 100000 for 100MBit
*/
DWORD NetStats::GetInterfaceBandwidth(int index)
{
	POSITION pos = Bandwidths.FindIndex(index);
	if (pos == NULL)
		return 0;
	else 
		return Bandwidths.GetAt(pos) / 8;
}

/*
	Sometime it is nice to know, how much traffic has a specific interface sent and received
*/

DWORD NetStats::GetInterfaceTotalTraffic(int index, int CurrentTrafficType)
{
	DWORD totaltraffic = 0.0;
	if ( CurrentTrafficType == AllTraffic )
	{
		POSITION pos = TotalTraffics.FindIndex(index);
		if (pos != NULL)
		{
			totaltraffic = TotalTraffics.GetAt(pos); 
			if (totaltraffic == 0.0)
			{
				GetTraffic(index, CurrentTrafficType);
				pos = TotalTraffics.FindIndex(index);
				if (pos != NULL)
				{
					totaltraffic = TotalTraffics.GetAt(pos); 
				}
			}
		}
	}
	else if ( CurrentTrafficType == IncomingTraffic )
	{
		POSITION pos = TotalIncomingTraffics.FindIndex(index);
		if (pos != NULL)
		{
			totaltraffic = TotalIncomingTraffics.GetAt(pos); 
			if (totaltraffic == 0.0)
			{
				GetTraffic(index, CurrentTrafficType);
				pos = TotalIncomingTraffics.FindIndex(index);
				if (pos != NULL)
				{
					totaltraffic = TotalIncomingTraffics.GetAt(pos); 
				}
			}
		}
	}
	else if ( CurrentTrafficType == OutGoingTraffic )
	{
		POSITION pos = TotalOutGoingTraffics.FindIndex(index);
		if (pos != NULL)
		{
			totaltraffic = TotalOutGoingTraffics.GetAt(pos); 
			if (totaltraffic == 0.0)
			{
				GetTraffic(index, CurrentTrafficType);
				pos = TotalOutGoingTraffics.FindIndex(index);
				if (pos != NULL)
				{
					totaltraffic = TotalOutGoingTraffics.GetAt(pos); 
				}
			}
		}
	}

	return totaltraffic;
}

CString NetStats::GetAllTotalBytes()
{
	return strAllTotalTraffic;
}

CString NetStats::GetAllBytesIn()
{
	return strAllIncomingTraffic;
}

CString NetStats::GetAllBytesOut()
{
	return strAllOutGoingTraffic;
}

CString NetStats::GetTotalBytesPerSec()
{
	return strCurrentTotalTraffic;
}

CString NetStats::GetBytesPerSecIn()
{
	return strCurrentIncomingTraffic;
}

CString NetStats::GetBytesPerSecOut()
{
	return strCurrentOutGoingTraffic;
}

CString NetStats::GetMyInterfaceName()
{
	return strMyInterfaceName;
}

BOOL NetStats::IsConnAvailable()
{
	BOOL isOnline = TRUE;

// If you have installed the Microsoft Platform SDK be happy, because you can use the feature
// added below..
#ifdef _I_HAVE_PLATFORM_SDK_INSTALLED_		

	DWORD flag, reserved;
	BOOL erg = TRUE;
	flag = 0;//INTERNET_CONNECTION_OFFLINE ;
	reserved = 0;
	TCHAR connectionname[1024];
	//	erg = InternetGetConnectedState(&flag, reserved);
	erg =  InternetGetConnectedStateEx(	&flag,	//OUT LPDWORD lpdwFlags,
										(LPTSTR)&connectionname,//OUT LPTSTR lpszConnectionName,
										1024,//IN DWORD dwNameLen,
										0//IN DWORD dwReserved
										);
		
	isOnline = erg;

#endif

	return isOnline;
}


